import cv
import os
from PIL import Image
from math import floor, ceil

from Utilerias import Utilerias
from Properties import Properties
from PuntosBiometricos import PuntosBiometricos
from Entrenamiento import Entrenamiento

def cargarImagenCV(path_imagen):
	return cv.LoadImage(obtenerPath(path_imagen), 0)

def cargarImagenPIL(path):
	return Image.open(path)

def obtenerPath(nombre):
	path = os.path.dirname(os.path.realpath(__file__))
	path = os.path.join(path, nombre)
	return path

def pintarEnImagen(path_imagen, lista):
	imagen = cargarImagenCV(path_imagen)
	for(x,y,ancho,alto),n in lista:
		cv.Rectangle(imagen, (x,y), (x+ancho, y+alto), 255)
	cv.SaveImage(path_imagen, imagen)
	return cv.LoadImage(obtenerPath(path_imagen), 0)

def recortarRostro(path_imagen, lista):
	imagen = cargarImagenCV(path_imagen)
	for (x,y, ancho, alto),n in lista:
		cv.SetImageROI(imagen, (x, y, ancho, alto))
		cv.SaveImage(path_imagen, imagen)

def obtenerCentro(lista):
	print "en obtener centro, lista len " ,len(lista)
	puntos = lista[0]
	noSeQueEs = lista[1]
	x,y,ancho,alto = puntos
	centro = (int(x) + int((floor(ancho / 2.0))), int(y) + int((floor( alto / 2.0 ))))
	return centro

def pintarLinea(path, ptos):
	imagen = cargarImagenCV(path)
	a = ptos[0]
	b = ptos[1]
	cv.Line(imagen, a, b, 255)
	cv.SaveImage(path, imagen)

def CargarImagenes(original, ut, p):
	original = ut.tomarFoto(original)
	ori = cargarImagenCV(original)
	pathEntrada = p.path_medidas
	pathSalida  = p.path_resultado

	cp1 = cv.CreateImage((ori.width,ori.height), cv.IPL_DEPTH_8U, ori.nChannels)
	cv.Copy(ori, cp1)
	cv.SaveImage(pathEntrada, cp1)

	im = cargarImagenCV(pathEntrada)

	copia = cv.CreateImage((im.width,im.height), cv.IPL_DEPTH_8U, im.nChannels)
	cv.Copy(im, copia)
	cv.SaveImage(pathSalida, copia)

	return original, pathEntrada, pathSalida

def ordenaLista(lista, elementos):
	lista.sort(key=lambda x: x[1])
	if len(lista) > elementos:
		sobrante = len(lista) - elementos
		for i in range(sobrante):
			lista.pop(0)
	return lista

def buscarPuntosBiometricos(im, io, utils, prop):
	pb = PuntosBiometricos(prop, utils)
	puntos_rostro = utils.buscarElemento(im, prop.cascada_rostro)
	recortarRostro(im, puntos_rostro)
	recortarRostro(io, puntos_rostro)

	#PARA REDIMENSIONAR LA IMAGEN
	imed = cargarImagenCV(im)
	isal = cargarImagenCV(io)
	red_medidas = cv.CreateImage( ( prop.imagen_x, prop.imagen_y ), imed.depth, imed.nChannels)
	cv.Resize(imed, red_medidas, interpolation=cv.CV_INTER_CUBIC)
	cv.Resize(isal, red_medidas, interpolation=cv.CV_INTER_CUBIC)
	cv.SaveImage(im, red_medidas)
	cv.SaveImage(io, red_medidas)

	imx = cargarImagenCV(im)
	tam = cv.GetSize(imx)
	
	pb.mapa_rostro = puntos_rostro

	puntos_ojos = utils.buscarElemento(im, prop.cascada_ojos)
	puntos_ojos = ordenaLista(puntos_ojos, 2)
	pb.mapa_ojos = puntos_ojos

	puntos_nariz = utils.buscarElemento(im, prop.cascada_nariz)
	puntos_nariz = ordenaLista(puntos_nariz, 1)
	pb.mapa_nariz = puntos_nariz

	puntos_boca = utils.buscarElemento(im, prop.cascada_boca)
	puntos_boca = ordenaLista(puntos_boca, 1)
	pb.mapa_boca = puntos_boca

	return pb

def entrenar():
	prop = Properties()
	utils = Utilerias()
	entrenamiento = Entrenamiento(utils, prop)
	entrenamiento.entrenar()

def validar():
	#cargarPropiedades y utilerias
	prop = Properties()
	utils = Utilerias()
	entrenamiento = Entrenamiento(utils, prop)

	iOriginal, iMedidas, iSalidas = utils.generarArchivosDeTrabajo(prop.path_original, prop, 1)
	#obtener puntos biometricos
	puntosBiometricos = buscarPuntosBiometricos(iMedidas, iSalidas, utils, prop)
	puntosBiometricos.UbicaCentros()
	puntosBiometricos.calculaDistancias()
	puntosBiometricos.pintarImagen(cargarImagenCV(iSalidas))
	#puntosBiometricos.info()
	ArrayFinal = puntosBiometricos.generaLinea()
	cad = ''

	f = open(prop.archivo_valores, 'r')
	while True:
		params = f.readline()
		if not params: 
			break
		print "params:", params
		cad += params
	f.close()

	print "puntos finales de usuario detectado: "
	print ArrayFinal

	print "parametros almacenados para el usuario"
	print cad

	#entrenamiento.entrenar()
	#Mostrar resultado, (si pintamos)
	im_p = cargarImagenPIL(iSalidas)
	im_p.show()

def main():
	validar()
	#entrenar()

main()
