from math import ceil, floor, sqrt
from PIL import Image, ImageDraw
import cv

class PuntosBiometricos():

	def __init__ (self, prop, utils):
		self.prop  = prop
		self.utils = utils

		# tam de imagen
		self.tam_imagen = (130,130)

		#mapas de cada elemento
		self.mapa_rostro = None
		self.mapa_ojos   = None
		self.mapa_nariz  = None
		self.mapa_boca   = None

		#centros
		self.centro_rostro        = None
		self.centro_ojo_izquierdo = None
		self.centro_ojo_derecho   = None 
		self.centro_nariz         = None
		self.centro_boca          = None

		self.centro_extremo_ojo_derecho   = None
		self.centro_extremo_ojo_izquierdo = None

		self.centro_interno_ojo_derecho   = None
		self.centro_interno_ojo_izquierdo = None 

		self.centro_boca_izquierdo = None
		self.centro_boca_derecho   = None

		#Medias
		self.media_x_rostro  = 0
		self.media_y_rostro  = 0
		self.media_x_ojo_der = 0
		self.media_y_ojo_der = 0
		self.media_x_ojo_izq = 0
		self.media_y_ojo_izq = 0
		self.media_x_nariz   = 0
		self.media_y_nariz   = 0
		self.media_x_boca    = 0
		self.media_y_boca    = 0

		#distancias
		self.d_ojos_centro         = 0
		self.d_ojos_extremos       = 0
		self.d_ojo_izquierdo_nariz = 0
		self.d_ojo_derecho_nariz   = 0
		self.d_ojo_izquierdo_boca  = 0
		self.d_ojo_derecho_boca    = 0 
		self.d_boca                = 0
		self.d_nariz_boca          = 0


	def pintarImagen(self, imagen):
		cv.Line(imagen, self.centro_ojo_izquierdo, self.centro_ojo_derecho, 255)
		cv.Line(imagen, self.centro_extremo_ojo_derecho, self.centro_boca_derecho, 255)
		cv.Line(imagen, self.centro_extremo_ojo_izquierdo, self.centro_boca_izquierdo, 255)
		cv.Line(imagen, self.centro_interno_ojo_izquierdo, self.centro_boca, 255)
		cv.Line(imagen, self.centro_interno_ojo_derecho, self.centro_boca, 255)
		cv.Line(imagen, self.centro_ojo_derecho, self.centro_nariz, 255)
		cv.Line(imagen, self.centro_ojo_izquierdo, self.centro_nariz, 255)
		cv.Line(imagen, self.centro_boca_derecho, self.centro_boca_izquierdo, 255)
		cv.Line(imagen, self.centro_nariz, self.centro_boca, 255)
		cv.SaveImage(self.prop.path_resultado, imagen)

	def calculaDistancias(self):
		self.d_ojos_centro         = self.dameDistancia(self.centro_ojo_izquierdo, self.centro_ojo_derecho)
		self.d_ojos_extremos       = self.dameDistancia(self.centro_extremo_ojo_izquierdo, self.centro_extremo_ojo_derecho)
		self.d_ojo_izquierdo_nariz = self.dameDistancia(self.centro_ojo_izquierdo, self.centro_nariz)
		self.d_ojo_derecho_nariz   = self.dameDistancia(self.centro_ojo_derecho, self.centro_nariz)
		self.d_ojo_izquierdo_boca  = self.dameDistancia(self.centro_extremo_ojo_izquierdo, self.centro_boca_izquierdo)
		self.d_ojo_derecho_boca    = self.dameDistancia(self.centro_extremo_ojo_derecho, self.centro_boca_derecho)
		self.d_boca                = self.dameDistancia(self.centro_boca_izquierdo, self.centro_boca_derecho) 
		self.d_nariz_boca          = self.dameDistancia(self.centro_nariz, self.centro_boca)

	def info(self):
		print "Distancias"
		print "distancia entre el centro de los ojos:   ", self.d_ojos_centro
		print "distancia entre los extemos de los ojos: ", self.d_ojos_extremos
		print "distancia entre ojo izquierdo y nariz:   ", self.d_ojo_izquierdo_nariz
		print "distancia entre ojo derecho y nariz:     ", self.d_ojo_derecho_nariz
		print "distancia entre ojo izquierdo y boca:    ", self.d_ojo_izquierdo_boca
		print "distancia entre ojo derecho y boca:      ", self.d_ojo_derecho_boca
		print "distancia ancho de la boca:              ", self.d_boca
		print "distancia entre nariz y boca:            ", self.d_nariz_boca

	def generaLinea(self):
		linea = str(self.d_ojos_centro)+","+str(self.d_ojos_extremos)+","+str(self.d_ojo_izquierdo_nariz)+","+str(self.d_ojo_derecho_nariz)+","+str(self.d_ojo_izquierdo_boca)+","+str(self.d_ojo_derecho_boca)+","+str(self.d_boca)+","+str(self.d_nariz_boca)
		return linea

	def dameDistancia(self, a, b):
		distancia = sqrt((a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2)
		return self.normaliza(distancia)

	def normaliza(self, n):
		if n - int(n) > 0.5:
			n = int(ceil(n))
		else:
			n = int(floor(n))
		return n

	def mostrar(self):
		print "Puntos Biometricos"
		print "mapa rostro: ", self.mapa_rostro
		print "mapa ojos:   ", self.mapa_ojos
		print "mapa nariz:  ", self.mapa_nariz
		print "mapa boca:   ", self.mapa_boca 
		print "CENTROS"
		print "rostro:  ", self.centro_rostro
		print "ojo izq: ", self.centro_ojo_izquierdo
		print "ojo der: ", self.centro_ojo_derecho
		print "nariz:   ", self.centro_nariz
		print "boca:    ", self.centro_boca  

	def UbicaCentros(self):
		#centro rostro
		(xr, yr, anchor, altor),n = self.mapa_rostro[0]
		self.media_x_rostro = self.normaliza(anchor / 2.0)
		self.media_y_rostro = self.normaliza(altor / 2.0)
		self.centro_rostro = (xr + self.media_x_rostro , yr + self.media_y_rostro)

		#centro de ojos
		(a,b,c,d),e = self.mapa_ojos[0]
		(f,g,h,i),k = self.mapa_ojos[1]

		if a < f:
			(xoi, yoi, woi, hoi), noi  = self.mapa_ojos[0]
			(xod, yod, wod, hod), nod  = self.mapa_ojos[1]
		else:
			(xoi, yoi, woi, hoi), noi  = self.mapa_ojos[1]
			(xod, yod, wod, hod), nod  = self.mapa_ojos[0]

		self.media_x_ojo_izq = self.normaliza(woi / 2.0)
		self.media_y_ojo_izq = self.normaliza(hoi / 2.0)

		self.media_x_ojo_der = self.normaliza(wod / 2.0)
		self.media_y_ojo_der = self.normaliza(hod / 2.0)

		self.centro_ojo_izquierdo  = (xoi + self.media_x_ojo_izq , yoi + self.media_y_ojo_izq)
		self.centro_ojo_derecho    = (xod + self.media_x_ojo_der , yod + self.media_y_ojo_der)

		self.centro_extremo_ojo_izquierdo = (self.centro_ojo_izquierdo[0] - self.media_x_ojo_izq, self.centro_ojo_izquierdo[1])
		self.centro_extremo_ojo_derecho   = (self.centro_ojo_derecho[0] + self.media_x_ojo_der, self.centro_ojo_derecho[1])

		self.centro_interno_ojo_derecho   = (self.centro_ojo_derecho[0] - self.media_x_ojo_der, self.centro_ojo_derecho[1])
		self.centro_interno_ojo_izquierdo = (self.centro_ojo_izquierdo[0] + self.media_x_ojo_izq, self.centro_ojo_izquierdo[1])

		#centro de nariz
		(xn, yn, wn, hn),nn = self.mapa_nariz[0]
		self.media_x_nariz  = self.normaliza(wn / 2.0)
		self.media_y_nariz  = self.normaliza(hn / 2.0)
		self.centro_nariz   = (xn + self.media_x_nariz, yn + self.media_y_nariz)

		#centro de boca
		(xb, yb, wb, hb), nb = self.mapa_boca[0]
		self.media_x_boca  = self.normaliza(wb / 2.0)
		self.media_y_boca  = self.normaliza(hb / 2.0)
		self.centro_boca   = (xb + self.media_x_boca, yb + self.media_y_boca)

		self.centro_boca_izquierdo = (self.centro_boca[0] - self.media_x_boca, self.centro_boca[1])
		self.centro_boca_derecho   = (self.centro_boca[0] + self.media_x_boca, self.centro_boca[1])
